@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2001, 2002, 2003, 2004, 2009, 2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2001, 2002, 2003, 2004, 2009, 2011 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -213,11 +213,6 @@ struct GNUNET_DHT_ClientPutMessage
    */
   uint32_t desired_replication_level GNUNET_PACKED;
 
-  /**
-   * Unique ID for the PUT message.
-   */
-  uint64_t unique_id GNUNET_PACKED;
-
   /**
    * How long should this data persist?
    */
@@ -233,30 +228,6 @@ struct GNUNET_DHT_ClientPutMessage
 };
 
 
-/**
- * Message to confirming receipt of PUT, sent from DHT service to clients.
- */
-struct GNUNET_DHT_ClientPutConfirmationMessage
-{
-  /**
-   * Type: #GNUNET_MESSAGE_TYPE_DHT_CLIENT_PUT_OK
-   */
-  struct GNUNET_MessageHeader header;
-
-  /**
-   * Always zero.
-   */
-  uint32_t reserved GNUNET_PACKED;
-
-  /**
-   * Unique ID from the PUT message that is being confirmed.
-   */
-  uint64_t unique_id GNUNET_PACKED;
-
-};
-
-
-
 /**
  * Message to monitor put requests going through peer, DHT service -> clients.
  */
