@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2002, 2003, 2004, 2006, 2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2002, 2003, 2004, 2006, 2013 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -76,9 +76,9 @@ main (int argc, char *argv[])
           GNUNET_STRINGS_relative_time_to_string (GNUNET_TIME_absolute_get_duration (start),
 						  GNUNET_YES));
   GAUGER ("NSE", "Proof-of-work hashing",
-          1024 / (1 +
+          1024.0 / (1.0 +
 		  GNUNET_TIME_absolute_get_duration
-		  (start).rel_value_us / 1000LL), "hashes/ms");
+		  (start).rel_value_us / 1000.0), "hashes/ms");
   return 0;
 }
 
