@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2009 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2009 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 /**
  * @file statistics/test_statistics_api_loop.c
@@ -31,7 +31,10 @@ static struct GNUNET_STATISTICS_Handle *h;
 
 
 static int
-check_1 (void *cls, const char *subsystem, const char *name, uint64_t value,
+check_1 (void *cls,
+	 const char *subsystem,
+	 const char *name,
+	 uint64_t value,
          int is_persistent)
 {
   GNUNET_assert (0 == strcmp (name, "test-0"));
@@ -42,7 +45,8 @@ check_1 (void *cls, const char *subsystem, const char *name, uint64_t value,
 
 
 static void
-next (void *cls, int success)
+next (void *cls,
+      int success)
 {
   int *ok = cls;
 
@@ -53,7 +57,9 @@ next (void *cls, int success)
 
 
 static void
-run (void *cls, char *const *args, const char *cfgfile,
+run (void *cls,
+     char *const *args,
+     const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
   unsigned int i;
@@ -62,15 +68,15 @@ run (void *cls, char *const *args, const char *cfgfile,
   h = GNUNET_STATISTICS_create ("test-statistics-api-loop", cfg);
   for (i = 0; i < ROUNDS; i++)
   {
-    GNUNET_snprintf (name, sizeof (name), "test-%d", i % 256);
+    GNUNET_snprintf (name, sizeof (name), "test-%d", i % 32);
     GNUNET_STATISTICS_set (h, name, i, GNUNET_NO);
-    GNUNET_snprintf (name, sizeof (name), "test-%d", i % 128);
+    GNUNET_snprintf (name, sizeof (name), "test-%d", i % 16);
     GNUNET_STATISTICS_update (h, name, 1, GNUNET_NO);
   }
   i = 0;
   GNUNET_break (NULL !=
                 GNUNET_STATISTICS_get (h, NULL, "test-0",
-                                       GNUNET_TIME_UNIT_MINUTES, &next,
+                                       &next,
                                        &check_1, cls));
 }
 
