@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2012 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -47,12 +47,12 @@ static struct GNUNET_DISK_PipeHandle *sigpipe;
  * process died) or when user presses CTRL-C.
  *
  * @param cls closure, NULL
- * @param tc scheduler context
  */
 static void
-maint_child_death (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+maint_child_death (void *cls)
 {
   enum GNUNET_OS_ProcessStatusType type;
+
   if ( (GNUNET_OK !=
 	GNUNET_OS_process_status (p, &type, &exit_code)) ||
        (type != GNUNET_OS_PROCESS_EXITED) )
@@ -77,16 +77,18 @@ run (void *cls, char *const *args, const char *cfgfile,
   const char *slash;
   char *subsystem;
   char *program;
-  GNUNET_SCHEDULER_TaskIdentifier rt;
+  struct GNUNET_SCHEDULER_Task * rt;
 
   if (NULL == (uri = args[0]))
   {
-    fprintf (stderr, _("No URI specified on command line\n"));
+    fprintf (stderr,
+	     _("No URI specified on command line\n"));
     return;
   }
   if (0 != strncasecmp ("gnunet://", uri, strlen ("gnunet://")))
   {
-    fprintf (stderr, _("Invalid URI: does not start with `%s'\n"),
+    fprintf (stderr,
+	     _("Invalid URI: does not start with `%s'\n"),
 	     "gnunet://");
     return;
   }
