@@ -1,27 +1,32 @@
 /*
      This file is part of GNUnet.
-     (C) 2006-2014 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2006-2018 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
  * @file src/util/os_installation.c
  * @brief get paths used by the program
  * @author Milan
+ * @author Christian Fuchs
+ * @author Christian Grothoff
+ * @author Matthias Wachs
+ * @author Heikki Lindholm
+ * @author LRN
  */
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -39,9 +44,64 @@
 #endif
 
 
-#define LOG(kind,...) GNUNET_log_from (kind, "util", __VA_ARGS__)
+#define LOG(kind,...) GNUNET_log_from (kind, "util-os-installation", __VA_ARGS__)
 
-#define LOG_STRERROR_FILE(kind,syscall,filename) GNUNET_log_from_strerror_file (kind, "util", syscall, filename)
+#define LOG_STRERROR_FILE(kind,syscall,filename) GNUNET_log_from_strerror_file (kind, "util-os-installation", syscall, filename)
+
+
+/**
+ * Default project data used for installation path detection
+ * for GNUnet (core).
+ */
+static const struct GNUNET_OS_ProjectData default_pd = {
+  .libname = "libgnunetutil",
+  .project_dirname = "gnunet",
+  .binary_name = "gnunet-arm",
+  .env_varname = "GNUNET_PREFIX",
+  .base_config_varname = "GNUNET_BASE_CONFIG",
+  .bug_email = "gnunet-developers@gnu.org",
+  .homepage = "http://www.gnu.org/s/gnunet/",
+  .config_file = "gnunet.conf",
+  .user_config_file = "~/.config/gnunet.conf",
+};
+
+/**
+ * Which project data do we currently use for installation
+ * path detection? Never NULL.
+ */
+static const struct GNUNET_OS_ProjectData *current_pd = &default_pd;
+
+/**
+ * Return default project data used by 'libgnunetutil' for GNUnet.
+ */
+const struct GNUNET_OS_ProjectData *
+GNUNET_OS_project_data_default (void)
+{
+  return &default_pd;
+}
+
+
+/**
+ * @return current project data.
+ */
+const struct GNUNET_OS_ProjectData *
+GNUNET_OS_project_data_get ()
+{
+  return current_pd;
+}
+
+
+/**
+ * Setup OS subsystem with project data.
+ *
+ * @param pd project data used to determine paths
+ */
+void
+GNUNET_OS_init (const struct GNUNET_OS_ProjectData *pd)
+{
+  GNUNET_assert (NULL != pd);
+  current_pd = pd;
+}
 
 
 #if LINUX
@@ -66,7 +126,8 @@ get_path_from_proc_maps ()
   {
     if ((1 ==
          SSCANF (line, "%*x-%*x %*c%*c%*c%*c %*x %*2x:%*2x %*u%*[ ]%1023s", dir)) &&
-        (NULL != (lgu = strstr (dir, "libgnunetutil"))))
+        (NULL != (lgu = strstr (dir,
+                                current_pd->libname))))
     {
       lgu[0] = '\0';
       FCLOSE (f);
@@ -89,24 +150,37 @@ get_path_from_proc_exe ()
   char fn[64];
   char lnk[1024];
   ssize_t size;
+  char *lep;
 
-  GNUNET_snprintf (fn, sizeof (fn), "/proc/%u/exe", getpid ());
-  size = readlink (fn, lnk, sizeof (lnk) - 1);
+  GNUNET_snprintf (fn,
+		   sizeof (fn),
+		   "/proc/%u/exe",
+		   getpid ());
+  size = readlink (fn,
+		   lnk,
+		   sizeof (lnk) - 1);
   if (size <= 0)
   {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_ERROR, "readlink", fn);
+    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_ERROR,
+		       "readlink",
+		       fn);
     return NULL;
   }
-  GNUNET_assert (size < sizeof (lnk));
+  GNUNET_assert ( ((size_t) size) < sizeof (lnk));
   lnk[size] = '\0';
   while ((lnk[size] != '/') && (size > 0))
     size--;
+  GNUNET_asprintf (&lep,
+                   "/%s/libexec/",
+                   current_pd->project_dirname);
   /* test for being in lib/gnunet/libexec/ or lib/MULTIARCH/gnunet/libexec */
-  if ( (size > strlen ("/gnunet/libexec/")) &&
-       (0 == strcmp ("/gnunet/libexec/",
-		     &lnk[size - strlen ("/gnunet/libexec/")])) )
-    size -= strlen ("gnunet/libexec/");
-  if ((size < 4) || (lnk[size - 4] != '/'))
+  if ( (((size_t) size) > strlen (lep)) &&
+       (0 == strcmp (lep,
+		     &lnk[size - strlen (lep)])) )
+    size -= strlen (lep) - 1;
+  GNUNET_free (lep);
+  if ( (size < 4) ||
+       (lnk[size - 4] != '/') )
   {
     /* not installed in "/bin/" -- binary path probably useless */
     return NULL;
@@ -127,7 +201,9 @@ static HINSTANCE dll_instance;
  * and hInstance saving.
  */
 BOOL WINAPI
-DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+DllMain (HINSTANCE hinstDLL,
+         DWORD fdwReason,
+         LPVOID lpvReserved)
 {
   switch (fdwReason)
   {
@@ -168,12 +244,15 @@ get_path_from_module_filename ()
   do
   {
     pathlen = pathlen * 2;
-    modulepath = GNUNET_realloc (modulepath, pathlen * sizeof (wchar_t));
+    modulepath = GNUNET_realloc (modulepath,
+                                 pathlen * sizeof (wchar_t));
     SetLastError (0);
-    real_pathlen = GetModuleFileNameW (dll_instance, modulepath, pathlen * sizeof (wchar_t));
+    real_pathlen = GetModuleFileNameW (dll_instance,
+                                       modulepath,
+                                       pathlen * sizeof (wchar_t));
   } while (real_pathlen >= pathlen && pathlen < 16*1024);
   if (real_pathlen >= pathlen)
-    GNUNET_abort ();
+    GNUNET_assert (0);
   /* To be safe */
   modulepath[real_pathlen] = '\0';
 
@@ -215,10 +294,10 @@ get_path_from_module_filename ()
   /* modulepath is GNUNET_PREFIX */
   u8_string = u16_to_u8 (modulepath, wcslen (modulepath), NULL, &u8_string_length);
   if (NULL == u8_string)
-    GNUNET_abort ();
+    GNUNET_assert (0);
 
   upath = GNUNET_malloc (u8_string_length + 1);
-  memcpy (upath, u8_string, u8_string_length);
+  GNUNET_memcpy (upath, u8_string, u8_string_length);
   upath[u8_string_length] = '\0';
 
   free (u8_string);
@@ -234,10 +313,12 @@ get_path_from_module_filename ()
  * Signature of the '_NSGetExecutablePath" function.
  *
  * @param buf where to write the path
- * @param number of bytes available in 'buf'
+ * @param number of bytes available in @a buf
  * @return 0 on success, otherwise desired number of bytes is stored in 'bufsize'
  */
-typedef int (*MyNSGetExecutablePathProto) (char *buf, size_t * bufsize);
+typedef int
+(*MyNSGetExecutablePathProto) (char *buf,
+                               size_t *bufsize);
 
 
 /**
@@ -294,7 +375,8 @@ get_path_from_dyld_image ()
   c = _dyld_image_count ();
   for (i = 0; i < c; i++)
   {
-    if (((const void *) _dyld_get_image_header (i)) != (const void *)&_mh_dylib_header)
+    if (((const void *) _dyld_get_image_header (i)) !=
+        ((const void *) &_mh_dylib_header) )
       continue;
     path = _dyld_get_image_name (i);
     if ( (NULL == path) || (0 == strlen (path)) )
@@ -376,7 +458,11 @@ get_path_from_GNUNET_PREFIX ()
 {
   const char *p;
 
-  if (NULL != (p = getenv ("GNUNET_PREFIX")))
+  if ( (NULL != current_pd->env_varname) &&
+       (NULL != (p = getenv (current_pd->env_varname))) )
+    return GNUNET_strdup (p);
+  if ( (NULL != current_pd->env_varname_alt) &&
+       (NULL != (p = getenv (current_pd->env_varname_alt))) )
     return GNUNET_strdup (p);
   return NULL;
 }
@@ -399,7 +485,8 @@ os_get_gnunet_path ()
   if (NULL != (ret = get_path_from_proc_maps ()))
     return ret;
   /* try path *first*, before /proc/exe, as /proc/exe can be wrong */
-  if (NULL != (ret = get_path_from_PATH ("gnunet-arm")))
+  if ( (NULL != current_pd->binary_name) &&
+       (NULL != (ret = get_path_from_PATH (current_pd->binary_name))) )
     return ret;
   if (NULL != (ret = get_path_from_proc_exe ()))
     return ret;
@@ -414,20 +501,20 @@ os_get_gnunet_path ()
   if (NULL != (ret = get_path_from_NSGetExecutablePath ()))
     return ret;
 #endif
-  if (NULL != (ret = get_path_from_PATH ("gnunet-arm")))
+  if ( (NULL != current_pd->binary_name) &&
+       (NULL != (ret = get_path_from_PATH (current_pd->binary_name))) )
     return ret;
   /* other attempts here */
   LOG (GNUNET_ERROR_TYPE_ERROR,
        _("Could not determine installation path for %s.  Set `%s' environment variable.\n"),
-       "GNUnet", "GNUNET_PREFIX");
+       current_pd->project_dirname,
+       current_pd->env_varname);
   return NULL;
 }
 
 
 /**
  * @brief get the path to current app's bin/
- * @author Milan
- *
  * @return a pointer to the executable path, or NULL on error
  */
 static char *
@@ -455,14 +542,13 @@ os_get_exec_path ()
 /**
  * @brief get the path to a specific GNUnet installation directory or,
  * with #GNUNET_OS_IPK_SELF_PREFIX, the current running apps installation directory
- * @author Milan
  * @return a pointer to the dir path (to be freed by the caller)
  */
 char *
 GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
 {
   size_t n;
-  const char *dirname;
+  char *dirname;
   char *execpath = NULL;
   char *tmp;
   char *multiarch;
@@ -533,21 +619,23 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
   {
   case GNUNET_OS_IPK_PREFIX:
   case GNUNET_OS_IPK_SELF_PREFIX:
-    dirname = DIR_SEPARATOR_STR;
+    dirname = GNUNET_strdup (DIR_SEPARATOR_STR);
     break;
   case GNUNET_OS_IPK_BINDIR:
-    dirname = DIR_SEPARATOR_STR "bin" DIR_SEPARATOR_STR;
+    dirname = GNUNET_strdup (DIR_SEPARATOR_STR "bin" DIR_SEPARATOR_STR);
     break;
   case GNUNET_OS_IPK_LIBDIR:
     if (isbasedir)
     {
       GNUNET_asprintf (&tmp,
-                       "%s%s%s%s%s",
+                       "%s%s%s%s%s%s%s",
                        execpath,
                        DIR_SEPARATOR_STR "lib",
                        (NULL != multiarch) ? DIR_SEPARATOR_STR : "",
                        (NULL != multiarch) ? multiarch : "",
-                       DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR);
+                       DIR_SEPARATOR_STR,
+                       current_pd->project_dirname,
+                       DIR_SEPARATOR_STR);
       if (GNUNET_YES ==
           GNUNET_DISK_directory_test (tmp, GNUNET_YES))
       {
@@ -556,10 +644,12 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
       }
       GNUNET_free (tmp);
       tmp = NULL;
+      dirname = NULL;
       if (4 == sizeof (void *))
       {
-	dirname =
-	  DIR_SEPARATOR_STR "lib32" DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR;
+	GNUNET_asprintf (&dirname,
+                         DIR_SEPARATOR_STR "lib32" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR,
+                         current_pd->project_dirname);
 	GNUNET_asprintf (&tmp,
                          "%s%s",
                          execpath,
@@ -567,8 +657,9 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
       }
       if (8 == sizeof (void *))
       {
-	dirname =
-	  DIR_SEPARATOR_STR "lib64" DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR;
+        GNUNET_asprintf (&dirname,
+                         DIR_SEPARATOR_STR "lib64" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR,
+                         current_pd->project_dirname);
 	GNUNET_asprintf (&tmp,
                          "%s%s",
                          execpath,
@@ -580,34 +671,38 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
             GNUNET_DISK_directory_test (tmp, GNUNET_YES)) )
       {
         GNUNET_free (execpath);
+        GNUNET_free_non_null (dirname);
         return tmp;
       }
       GNUNET_free (tmp);
+      GNUNET_free_non_null (dirname);
     }
-    dirname = DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR;
+    GNUNET_asprintf (&dirname,
+                     DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR,
+                     current_pd->project_dirname);
     break;
   case GNUNET_OS_IPK_DATADIR:
-    dirname =
-        DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR;
+    GNUNET_asprintf (&dirname,
+                     DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR,
+                     current_pd->project_dirname);
     break;
   case GNUNET_OS_IPK_LOCALEDIR:
-    dirname =
-        DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "locale" DIR_SEPARATOR_STR;
+    dirname = GNUNET_strdup (DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "locale" DIR_SEPARATOR_STR);
     break;
   case GNUNET_OS_IPK_ICONDIR:
-    dirname =
-        DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "icons" DIR_SEPARATOR_STR;
+    dirname = GNUNET_strdup (DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "icons" DIR_SEPARATOR_STR);
     break;
   case GNUNET_OS_IPK_DOCDIR:
-    dirname =
-        DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "doc" DIR_SEPARATOR_STR \
-        "gnunet" DIR_SEPARATOR_STR;
+    GNUNET_asprintf (&dirname,
+                     DIR_SEPARATOR_STR "share" DIR_SEPARATOR_STR "doc" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR,
+                     current_pd->project_dirname);
     break;
   case GNUNET_OS_IPK_LIBEXECDIR:
     if (isbasedir)
     {
-      dirname =
-        DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR "libexec" DIR_SEPARATOR_STR;
+      GNUNET_asprintf (&dirname,
+                       DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR "libexec" DIR_SEPARATOR_STR,
+                       current_pd->project_dirname);
       GNUNET_asprintf (&tmp,
                        "%s%s%s%s",
                        execpath,
@@ -618,15 +713,17 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
           GNUNET_DISK_directory_test (tmp, GNUNET_YES))
       {
         GNUNET_free (execpath);
+        GNUNET_free (dirname);
         return tmp;
       }
       GNUNET_free (tmp);
       tmp = NULL;
+      dirname = NULL;
       if (4 == sizeof (void *))
       {
-	dirname =
-	  DIR_SEPARATOR_STR "lib32" DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR \
-	  "libexec" DIR_SEPARATOR_STR;
+        GNUNET_asprintf (&dirname,
+                         DIR_SEPARATOR_STR "lib32" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR "libexec" DIR_SEPARATOR_STR,
+                         current_pd->project_dirname);
 	GNUNET_asprintf (&tmp,
                          "%s%s",
                          execpath,
@@ -634,9 +731,9 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
       }
       if (8 == sizeof (void *))
       {
-	dirname =
-	  DIR_SEPARATOR_STR "lib64" DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR \
-	  "libexec" DIR_SEPARATOR_STR;
+	GNUNET_asprintf (&dirname,
+                         DIR_SEPARATOR_STR "lib64" DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR "libexec" DIR_SEPARATOR_STR,
+                         current_pd->project_dirname);
 	GNUNET_asprintf (&tmp,
                          "%s%s",
                          execpath,
@@ -647,14 +744,15 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
             GNUNET_DISK_directory_test (tmp, GNUNET_YES)) )
       {
         GNUNET_free (execpath);
+        GNUNET_free_non_null (dirname);
         return tmp;
       }
-
       GNUNET_free (tmp);
+      GNUNET_free_non_null (dirname);
     }
-    dirname =
-      DIR_SEPARATOR_STR "gnunet" DIR_SEPARATOR_STR \
-      "libexec" DIR_SEPARATOR_STR;
+    GNUNET_asprintf (&dirname,
+                     DIR_SEPARATOR_STR "%s" DIR_SEPARATOR_STR "libexec" DIR_SEPARATOR_STR,
+                     current_pd->project_dirname);
     break;
   default:
     GNUNET_free (execpath);
@@ -664,6 +762,7 @@ GNUNET_OS_installation_get_path (enum GNUNET_OS_InstallationPathKind dirkind)
                    "%s%s",
                    execpath,
                    dirname);
+  GNUNET_free (dirname);
   GNUNET_free (execpath);
   return tmp;
 }
@@ -685,7 +784,10 @@ GNUNET_OS_get_libexec_binary_path (const char *progname)
   char *binary;
 
   if ( (DIR_SEPARATOR == progname[0]) ||
-       (GNUNET_YES == GNUNET_STRINGS_path_is_absolute (progname, GNUNET_NO, NULL, NULL)) )
+       (GNUNET_YES ==
+        GNUNET_STRINGS_path_is_absolute (progname,
+                                         GNUNET_NO,
+                                         NULL, NULL)) )
     return GNUNET_strdup (progname);
   if (NULL != cache)
     libexecdir = cache;
@@ -703,10 +805,9 @@ GNUNET_OS_get_libexec_binary_path (const char *progname)
 
 
 /**
- * Check whether an executable exists and possibly
- * if the suid bit is set on the file.
- * Attempts to find the file using the current
- * PATH environment variable as a search path.
+ * Check whether an executable exists and possibly if the suid bit is
+ * set on the file.  Attempts to find the file using the current PATH
+ * environment variable as a search path.
  *
  * @param binary the name of the file to check.
  *        W32: must not have an .exe suffix.
@@ -720,7 +821,9 @@ GNUNET_OS_get_libexec_binary_path (const char *progname)
  *         #GNUNET_SYSERR on error (no such binary or not executable)
  */
 int
-GNUNET_OS_check_helper_binary (const char *binary, int check_suid, const char *params)
+GNUNET_OS_check_helper_binary (const char *binary,
+                               int check_suid,
+                               const char *params)
 {
   struct stat statbuf;
   char *p;
@@ -728,9 +831,13 @@ GNUNET_OS_check_helper_binary (const char *binary, int check_suid, const char *p
 #ifdef MINGW
   char *binaryexe;
 
-  GNUNET_asprintf (&binaryexe, "%s.exe", binary);
-  if ( (GNUNET_YES == GNUNET_STRINGS_path_is_absolute (binaryexe, GNUNET_NO,
-						       NULL, NULL)) ||
+  GNUNET_asprintf (&binaryexe,
+                   "%s.exe",
+                   binary);
+  if ( (GNUNET_YES ==
+        GNUNET_STRINGS_path_is_absolute (binaryexe,
+                                         GNUNET_NO,
+                                         NULL, NULL)) ||
        (0 == strncmp (binary, "./", 2)) )
     p = GNUNET_strdup (binaryexe);
   else
@@ -745,16 +852,24 @@ GNUNET_OS_check_helper_binary (const char *binary, int check_suid, const char *p
   }
   GNUNET_free (binaryexe);
 #else
-  if ( (GNUNET_YES == GNUNET_STRINGS_path_is_absolute (binary, GNUNET_NO,
-						       NULL, NULL)) ||
+  if ( (GNUNET_YES ==
+        GNUNET_STRINGS_path_is_absolute (binary,
+                                         GNUNET_NO,
+                                         NULL,
+                                         NULL)) ||
        (0 == strncmp (binary, "./", 2)) )
+  {
     p = GNUNET_strdup (binary);
+  }
   else
   {
     p = get_path_from_PATH (binary);
     if (NULL != p)
     {
-      GNUNET_asprintf (&pf, "%s/%s", p, binary);
+      GNUNET_asprintf (&pf,
+                       "%s/%s",
+                       p,
+                       binary);
       GNUNET_free (p);
       p = pf;
     }
@@ -767,9 +882,12 @@ GNUNET_OS_check_helper_binary (const char *binary, int check_suid, const char *p
          binary);
     return GNUNET_SYSERR;
   }
-  if (0 != ACCESS (p, X_OK))
+  if (0 != ACCESS (p,
+                   X_OK))
   {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING, "access", p);
+    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING,
+                       "access",
+                       p);
     GNUNET_free (p);
     return GNUNET_SYSERR;
   }
@@ -778,22 +896,31 @@ GNUNET_OS_check_helper_binary (const char *binary, int check_suid, const char *p
   {
     /* as we run as root, we don't insist on SUID */
     GNUNET_free (p);
-    return GNUNET_OK;
+    return GNUNET_YES;
   }
 #endif
-  if (0 != STAT (p, &statbuf))
+  if (0 != STAT (p,
+                 &statbuf))
   {
-    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING, "stat", p);
+    LOG_STRERROR_FILE (GNUNET_ERROR_TYPE_WARNING,
+                       "stat",
+                       p);
     GNUNET_free (p);
     return GNUNET_SYSERR;
   }
-  if (check_suid){
+  if (check_suid)
+  {
 #ifndef MINGW
-    if ((0 != (statbuf.st_mode & S_ISUID)) && (0 == statbuf.st_uid))
+    (void) params;
+    if ( (0 != (statbuf.st_mode & S_ISUID)) &&
+         (0 == statbuf.st_uid) )
     {
       GNUNET_free (p);
       return GNUNET_YES;
     }
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                _("Binary `%s' exists, but is not SUID\n"),
+                p);
     /* binary exists, but not SUID */
 #else
     STARTUPINFO start;
