@@ -1,21 +1,21 @@
 /*
  This file is part of GNUnet.
- (C) 2009 Christian Grothoff (and other contributing authors)
+ Copyright (C) 2009 GNUnet e.V.
 
- GNUnet is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published
- by the Free Software Foundation; either version 3, or (at your
- option) any later version.
+ GNUnet is free software: you can redistribute it and/or modify it
+ under the terms of the GNU Affero General Public License as published
+ by the Free Software Foundation, either version 3 of the License,
+ or (at your option) any later version.
 
  GNUnet is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- General Public License for more details.
+ Affero General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with GNUnet; see the file COPYING.  If not, write to the
- Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- Boston, MA 02111-1307, USA.
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+     SPDX-License-Identifier: AGPL3.0-or-later
  */
 /**
  * @file resolver/test_resolver_api.c
@@ -27,6 +27,9 @@
 #include "resolver.h"
 
 
+static int disable_rootserver_check;
+
+
 /**
  * Using DNS root servers to check gnunet's resolver service
  * a.root-servers.net <-> 198.41.0.4 is a fix 1:1 mapping that should not change over years
@@ -37,7 +40,9 @@
 
 
 static void
-check_hostname (void *cls, const struct sockaddr *sa, socklen_t salen)
+check_hostname (void *cls,
+                const struct sockaddr *sa,
+                socklen_t salen)
 {
   int *ok = cls;
 
@@ -46,13 +51,15 @@ check_hostname (void *cls, const struct sockaddr *sa, socklen_t salen)
     (*ok) &= ~8;
     return;
   }
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, _("Got IP address `%s' for our host.\n"),
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Got IP address `%s' for our host.\n",
               GNUNET_a2s (sa, salen));
 }
 
 
 static void
-check_localhost_num (void *cls, const char *hostname)
+check_localhost_num (void *cls,
+                     const char *hostname)
 {
   int *ok = cls;
 
@@ -60,13 +67,15 @@ check_localhost_num (void *cls, const char *hostname)
     return;
   if (0 == strcmp (hostname, "127.0.0.1"))
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Received correct hostname `%s'.\n",
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Received correct hostname `%s'.\n",
                 hostname);
     (*ok) &= ~4;
   }
   else
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Received invalid hostname `%s'.\n",
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Received invalid hostname `%s'.\n",
                 hostname);
     GNUNET_break (0);
   }
@@ -74,7 +83,8 @@ check_localhost_num (void *cls, const char *hostname)
 
 
 static void
-check_localhost (void *cls, const char *hostname)
+check_localhost (void *cls,
+                 const char *hostname)
 {
   int *ok = cls;
 
@@ -82,7 +92,8 @@ check_localhost (void *cls, const char *hostname)
     return;
   if (0 == strcmp (hostname, "localhost"))
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Received correct hostname `%s'.\n",
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Received correct hostname `%s'.\n",
                 hostname);
     (*ok) &= ~2;
   }
@@ -106,57 +117,25 @@ check_127 (void *cls, const struct sockaddr *sa, socklen_t salen)
   GNUNET_assert (sizeof (struct sockaddr_in) == salen);
   if (sai->sin_addr.s_addr == htonl (INADDR_LOOPBACK))
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Received correct address.\n");
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Received correct address.\n");
     (*ok) &= ~1;
   }
   else
   {
     char buf[INET_ADDRSTRLEN];
 
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Received incorrect address`%s'.\n",
-		inet_ntop (AF_INET, &sai->sin_addr, buf, sizeof (buf)));
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Received incorrect address `%s'.\n",
+		inet_ntop (AF_INET,
+                           &sai->sin_addr,
+                           buf,
+                           sizeof (buf)));
     GNUNET_break (0);
   }
 }
 
 
-static void
-check_local_fqdn (void *cls, const char *gnunet_fqdn)
-{
-  int result = 0;
-
-  struct hostent *host;
-  char hostname[GNUNET_OS_get_hostname_max_length () + 1];
-
-  if (0 != gethostname (hostname, sizeof (hostname) - 1))
-  {
-    GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR | GNUNET_ERROR_TYPE_BULK,
-                         "gethostname");
-    return;
-  }
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, _("Resolving our FQDN `%s'\n"),
-              hostname);
-  host = gethostbyname (hostname);
-  if (NULL == host)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                _("Could not resolve our FQDN: %s %u\n"), hstrerror (h_errno),
-                h_errno);
-    return;
-  }
-
-  GNUNET_assert (0 != host);
-
-  result = strcmp (host->h_name, gnunet_fqdn);
-  if (0 != result)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Local resolved and resolver resolved fqdns are not equal\n");
-  }
-  GNUNET_assert (0 == result);
-}
-
-
 static void
 check_rootserver_ip (void *cls, const struct sockaddr *sa, socklen_t salen)
 {
@@ -183,7 +162,8 @@ check_rootserver_ip (void *cls, const struct sockaddr *sa, socklen_t salen)
 
 
 static void
-check_rootserver_name (void *cls, const char *hostname)
+check_rootserver_name (void *cls,
+                       const char *hostname)
 {
   int *ok = cls;
 
@@ -193,16 +173,17 @@ check_rootserver_name (void *cls, const char *hostname)
   if (0 == strcmp (hostname, ROOTSERVER_NAME))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Received correct rootserver hostname `%s'.\n", hostname);
+                "Received correct rootserver hostname `%s'.\n",
+                hostname);
     (*ok) &= ~2;
   }
   else
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Received invalid rootserver hostname `%s', expected `%s'\n",
                 hostname,
                 ROOTSERVER_NAME);
-    GNUNET_break (0);
+    GNUNET_break (disable_rootserver_check);
   }
 }
 
@@ -217,6 +198,9 @@ run (void *cls, char *const *args, const char *cfgfile,
       GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 30);
   int count_ips = 0;
   char *own_fqdn;
+  const char *rootserver_name = ROOTSERVER_NAME;
+  struct hostent *rootserver;
+  struct in_addr rootserver_addr;
 
   memset (&sa, 0, sizeof (sa));
   sa.sin_family = AF_INET;
@@ -229,31 +213,31 @@ run (void *cls, char *const *args, const char *cfgfile,
    * Looking up our own fqdn
    */
   own_fqdn = GNUNET_RESOLVER_local_fqdn_get ();
-  check_local_fqdn (NULL, own_fqdn);
+  /* can't really check, only thing we can safely
+     compare against is our own identical logic... */
   GNUNET_free_non_null (own_fqdn);
 
   /*
    * Testing non-local DNS resolution
    * DNS rootserver to test: a.root-servers.net - 198.41.0.4
    */
-  const char *rootserver_name = ROOTSERVER_NAME;
-  struct hostent *rootserver;
 
   rootserver = gethostbyname (rootserver_name);
-  if (rootserver == NULL)
+  if (NULL == rootserver)
   {
     /* Error: resolving ip addresses does not work */
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 _("gethostbyname() could not lookup IP address: %s\n"),
                 hstrerror (h_errno));
     FPRINTF (stderr,
-             "%s", "System seems to be off-line, will not run all DNS tests\n");
+             "%s",
+             "System seems to be off-line, will not run all DNS tests\n");
     *ok = 0;                    /* mark test as passing anyway */
     return;
   }
 
   /* Counting returned IP addresses */
-  while (rootserver->h_addr_list[count_ips] != NULL)
+  while (NULL != rootserver->h_addr_list[count_ips])
     count_ips++;
   if (count_ips > 1)
   {
@@ -263,9 +247,9 @@ run (void *cls, char *const *args, const char *cfgfile,
   }
 
   /* Comparing to resolved address to the address the root name server should have */
-  if (strcmp
-      (inet_ntoa (*(struct in_addr *) rootserver->h_addr_list[0]),
-       ROOTSERVER_IP) != 0)
+  if (0 !=
+      strcmp (inet_ntoa (*(struct in_addr *) rootserver->h_addr_list[0]),
+              ROOTSERVER_IP))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "IP received and IP for root name server differ\n");
@@ -276,16 +260,16 @@ run (void *cls, char *const *args, const char *cfgfile,
   /* Resolve the same using GNUNET */
   GNUNET_RESOLVER_ip_get (ROOTSERVER_NAME, AF_INET, timeout,
                           &check_rootserver_ip, cls);
+  GNUNET_RESOLVER_ip_get (ROOTSERVER_NAME, AF_INET, timeout,
+                          &check_rootserver_ip, cls);
 
   /*
    * Success: forward lookups work as expected
    * Next step: reverse lookups
    */
-
-  struct in_addr rootserver_addr;
-
-  rootserver->h_name = "";
-  if (1 != inet_pton (AF_INET, ROOTSERVER_IP, &rootserver_addr))
+  if (1 != inet_pton (AF_INET,
+                      ROOTSERVER_IP,
+                      &rootserver_addr))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Could not transform root name server IP address\n");
@@ -293,22 +277,25 @@ run (void *cls, char *const *args, const char *cfgfile,
   }
 
   rootserver =
-      gethostbyaddr (&rootserver_addr, sizeof (rootserver_addr), AF_INET);
+      gethostbyaddr ((const void *) &rootserver_addr,
+                     sizeof (rootserver_addr),
+                     AF_INET);
   if (NULL == rootserver)
   {
     /* Error: resolving IP addresses does not work */
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "gethostbyaddr() could not lookup hostname: %s\n",
                 hstrerror (h_errno));
-    GNUNET_break (0);
+    disable_rootserver_check = GNUNET_YES;
   }
   else
   {
-    if (0 != strcmp (rootserver->h_name, ROOTSERVER_NAME))
+    if (0 != strcmp (rootserver->h_name,
+                     ROOTSERVER_NAME))
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Received hostname and hostname for root name server differ\n");
-      GNUNET_break (0);
+      disable_rootserver_check = GNUNET_YES;
     }
   }
 
