@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2001-2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2001-2013, 2018 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -25,8 +25,19 @@
  */
 #include "platform.h"
 #include "gnunet_util_lib.h"
+#if __STDC_NO_ATOMICS__
+#define ATOMIC
+#else
+#ifdef HAVE_STDATOMIC_H
+#include <stdatomic.h>
+#define ATOMIC _Atomic
+#else
+#define __STDC_NO_ATOMICS__ 1
+#define ATOMIC
+#endif
+#endif
 
-#define LOG(kind,...) GNUNET_log_from (kind, "util", __VA_ARGS__)
+#define LOG(kind,...) GNUNET_log_from (kind, "util-time", __VA_ARGS__)
 
 /**
  * Variable used to simulate clock skew.  Used for testing, never in production.
@@ -57,6 +68,46 @@ GNUNET_TIME_get_offset ()
 }
 
 
+/**
+ * Round a time value so that it is suitable for transmission
+ * via JSON encodings.
+ *
+ * @param at time to round
+ * @return #GNUNET_OK if time was already rounded, #GNUNET_NO if
+ *         it was just now rounded
+ */
+int
+GNUNET_TIME_round_abs (struct GNUNET_TIME_Absolute *at)
+{
+  if (at->abs_value_us == GNUNET_TIME_UNIT_FOREVER_ABS.abs_value_us)
+    return GNUNET_OK;
+  if (0 == at->abs_value_us % 1000000)
+    return GNUNET_OK;
+  at->abs_value_us -= at->abs_value_us % 1000000;
+  return GNUNET_NO;
+}
+
+
+/**
+ * Round a time value so that it is suitable for transmission
+ * via JSON encodings.
+ *
+ * @param rt time to round
+ * @return #GNUNET_OK if time was already rounded, #GNUNET_NO if
+ *         it was just now rounded
+ */
+int
+GNUNET_TIME_round_rel (struct GNUNET_TIME_Relative *rt)
+{
+  if (rt->rel_value_us == GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us)
+    return GNUNET_OK;
+  if (0 == rt->rel_value_us % 1000000)
+    return GNUNET_OK;
+  rt->rel_value_us -= rt->rel_value_us % 1000000;
+  return GNUNET_NO;
+}
+
+
 /**
  * Get the current time (works just as "time", just that we use the
  * unit of time that the cron-jobs use (and is 64 bit)).
@@ -108,6 +159,7 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_unit_ ()
 {
   static struct GNUNET_TIME_Relative one = { 1 };
+
   return one;
 }
 
@@ -119,6 +171,7 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_millisecond_ ()
 {
   static struct GNUNET_TIME_Relative one = { 1000 };
+
   return one;
 }
 
@@ -130,6 +183,7 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_second_ ()
 {
   static struct GNUNET_TIME_Relative one = { 1000 * 1000LL };
+
   return one;
 }
 
@@ -141,6 +195,7 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_minute_ ()
 {
   static struct GNUNET_TIME_Relative one = { 60 * 1000 * 1000LL };
+
   return one;
 }
 
@@ -152,6 +207,7 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_hour_ ()
 {
   static struct GNUNET_TIME_Relative one = { 60 * 60 * 1000 * 1000LL };
+
   return one;
 }
 
@@ -163,9 +219,11 @@ struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_get_forever_ ()
 {
   static struct GNUNET_TIME_Relative forever = { UINT64_MAX };
+
   return forever;
 }
 
+
 /**
  * Return "forever".
  */
@@ -176,6 +234,7 @@ GNUNET_TIME_absolute_get_forever_ ()
   return forever;
 }
 
+
 /**
  * Convert relative time to an absolute time in the
  * future.
@@ -308,7 +367,7 @@ GNUNET_TIME_absolute_get_difference (struct GNUNET_TIME_Absolute start,
  * Get the duration of an operation as the
  * difference of the current time and the given start time "whence".
  *
- * @return aborts if whence==FOREVER, 0 if whence > now, otherwise now-whence.
+ * @return 0 if whence > now, otherwise now-whence.
  */
 struct GNUNET_TIME_Relative
 GNUNET_TIME_absolute_get_duration (struct GNUNET_TIME_Absolute whence)
@@ -317,7 +376,6 @@ GNUNET_TIME_absolute_get_duration (struct GNUNET_TIME_Absolute whence)
   struct GNUNET_TIME_Relative ret;
 
   now = GNUNET_TIME_absolute_get ();
-  GNUNET_assert (whence.abs_value_us != UINT64_MAX);
   if (whence.abs_value_us > now.abs_value_us)
     return GNUNET_TIME_UNIT_ZERO;
   ret.rel_value_us = now.abs_value_us - whence.abs_value_us;
@@ -379,7 +437,7 @@ GNUNET_TIME_absolute_subtract (struct GNUNET_TIME_Absolute start,
  */
 struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_multiply (struct GNUNET_TIME_Relative rel,
-                               unsigned int factor)
+                               unsigned long long factor)
 {
   struct GNUNET_TIME_Relative ret;
 
@@ -387,7 +445,7 @@ GNUNET_TIME_relative_multiply (struct GNUNET_TIME_Relative rel,
     return GNUNET_TIME_UNIT_ZERO;
   if (rel.rel_value_us == GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us)
     return GNUNET_TIME_UNIT_FOREVER_REL;
-  ret.rel_value_us = rel.rel_value_us * (unsigned long long) factor;
+  ret.rel_value_us = rel.rel_value_us * factor;
   if (ret.rel_value_us / factor != rel.rel_value_us)
   {
     GNUNET_break (0);
@@ -397,6 +455,65 @@ GNUNET_TIME_relative_multiply (struct GNUNET_TIME_Relative rel,
 }
 
 
+/**
+ * Multiply relative time by a given floating-point factor.  The factor must be
+ * positive.
+ *
+ * @return FOREVER if rel=FOREVER or on overflow; otherwise rel*factor
+ */
+struct GNUNET_TIME_Relative
+relative_multiply_double (struct GNUNET_TIME_Relative rel,
+                          double factor)
+{
+  struct GNUNET_TIME_Relative out;
+  double m;
+
+  GNUNET_assert (0 <= factor);
+
+  if (0 == factor)
+    return GNUNET_TIME_UNIT_ZERO;
+  if (rel.rel_value_us == GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us)
+    return GNUNET_TIME_UNIT_FOREVER_REL;
+
+  m = ((double) rel.rel_value_us) * factor;
+
+  if (m >= (double) (GNUNET_TIME_UNIT_FOREVER_REL).rel_value_us)
+  {
+    GNUNET_break (0);
+    return GNUNET_TIME_UNIT_FOREVER_REL;
+  }
+
+  out.rel_value_us = (uint64_t) m;
+  return out;
+}
+
+
+/**
+ * Saturating multiply relative time by a given factor.
+ *
+ * @param rel some duration
+ * @param factor integer to multiply with
+ * @return FOREVER if rel=FOREVER or on overflow; otherwise rel*factor
+ */
+struct GNUNET_TIME_Relative
+GNUNET_TIME_relative_saturating_multiply (struct GNUNET_TIME_Relative rel,
+                                          unsigned long long factor)
+{
+  struct GNUNET_TIME_Relative ret;
+
+  if (0 == factor)
+    return GNUNET_TIME_UNIT_ZERO;
+  if (rel.rel_value_us == GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us)
+    return GNUNET_TIME_UNIT_FOREVER_REL;
+  ret.rel_value_us = rel.rel_value_us * factor;
+  if (ret.rel_value_us / factor != rel.rel_value_us)
+  {
+    return GNUNET_TIME_UNIT_FOREVER_REL;
+  }
+  return ret;
+}
+
+
 /**
  * Divide relative time by a given factor.
  *
@@ -406,14 +523,14 @@ GNUNET_TIME_relative_multiply (struct GNUNET_TIME_Relative rel,
  */
 struct GNUNET_TIME_Relative
 GNUNET_TIME_relative_divide (struct GNUNET_TIME_Relative rel,
-                             unsigned int factor)
+                             unsigned long long factor)
 {
   struct GNUNET_TIME_Relative ret;
 
   if ((0 == factor) ||
       (rel.rel_value_us == GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us))
     return GNUNET_TIME_UNIT_FOREVER_REL;
-  ret.rel_value_us = rel.rel_value_us / (unsigned long long) factor;
+  ret.rel_value_us = rel.rel_value_us / factor;
   return ret;
 }
 
@@ -524,7 +641,6 @@ GNUNET_TIME_relative_ntoh (struct GNUNET_TIME_RelativeNBO a)
 
   ret.rel_value_us = GNUNET_ntohll (a.rel_value_us__);
   return ret;
-
 }
 
 
@@ -620,7 +736,7 @@ GNUNET_TIME_year_to_time (unsigned int year)
   }
   t.tm_year = year - 1900;
   t.tm_mday = 1;
-  t.tm_mon = 1;
+  t.tm_mon = 0;
   t.tm_wday = 1;
   t.tm_yday = 1;
   tp = mktime (&t);
@@ -630,5 +746,175 @@ GNUNET_TIME_year_to_time (unsigned int year)
 }
 
 
+/**
+ * Randomized exponential back-off, starting at 1 ms
+ * and going up by a factor of 2+r, where 0 <= r <= 0.5, up
+ * to a maximum of the given threshold.
+ *
+ * @param r current backoff time, initially zero
+ * @param threshold maximum value for backoff
+ * @return the next backoff time
+ */
+struct GNUNET_TIME_Relative
+GNUNET_TIME_randomized_backoff(struct GNUNET_TIME_Relative rt, struct GNUNET_TIME_Relative threshold)
+{
+  double r = (rand() % 500) / 1000.0;
+  struct GNUNET_TIME_Relative t;
+
+  t = relative_multiply_double (GNUNET_TIME_relative_max (GNUNET_TIME_UNIT_MILLISECONDS,
+                                                          rt),
+                                2 + r);
+  return GNUNET_TIME_relative_min (threshold,
+                                   t);
+}
+
+
+/**
+ * Obtain the current time and make sure it is monotonically
+ * increasing.  Guards against systems without an RTC or
+ * clocks running backwards and other nasty surprises. Does
+ * not guarantee that the returned time is near the current
+ * time returned by #GNUNET_TIME_absolute_get().  Two
+ * subsequent calls (within a short time period) may return the
+ * same value. Persists the last returned time on disk to
+ * ensure that time never goes backwards. As a result, the
+ * resulting value can be used to check if a message is the
+ * "most recent" value and replays of older messages (from
+ * the same origin) would be discarded.
+ *
+ * @param cfg configuration, used to determine where to
+ *   store the time; user can also insist RTC is working
+ *   nicely and disable the feature
+ * @return monotonically increasing time
+ */
+struct GNUNET_TIME_Absolute
+GNUNET_TIME_absolute_get_monotonic (const struct GNUNET_CONFIGURATION_Handle *cfg)
+{
+  static const struct GNUNET_CONFIGURATION_Handle *last_cfg;
+  static struct GNUNET_TIME_Absolute last_time;
+  static struct GNUNET_DISK_MapHandle *map_handle;
+  static ATOMIC volatile uint64_t *map;
+  struct GNUNET_TIME_Absolute now;
+
+  now = GNUNET_TIME_absolute_get ();
+  if (last_cfg != cfg)
+  {
+    char *filename;
+
+    if (NULL != map_handle)
+    {
+      GNUNET_DISK_file_unmap (map_handle);
+      map_handle = NULL;
+    }
+    map = NULL;
+
+    last_cfg = cfg;
+    if ( (NULL != cfg) &&
+	 (GNUNET_OK ==
+	  GNUNET_CONFIGURATION_get_value_filename (cfg,
+						   "util",
+						   "MONOTONIC_TIME_FILENAME",
+						   &filename)) )
+    {
+      struct GNUNET_DISK_FileHandle *fh;
+
+      fh = GNUNET_DISK_file_open (filename,
+				  GNUNET_DISK_OPEN_READWRITE | GNUNET_DISK_OPEN_CREATE,
+				  GNUNET_DISK_PERM_USER_WRITE | GNUNET_DISK_PERM_GROUP_WRITE |
+				  GNUNET_DISK_PERM_USER_READ  | GNUNET_DISK_PERM_GROUP_READ);
+      if (NULL == fh)
+      {
+	GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+		    _("Failed to map `%s', cannot assure monotonic time!\n"),
+		    filename);
+      }
+      else
+      {
+	off_t size;
+
+	size = 0;
+	GNUNET_break (GNUNET_OK ==
+		      GNUNET_DISK_file_handle_size (fh,
+						    &size));
+	if (size < sizeof (*map))
+	{
+	  struct GNUNET_TIME_AbsoluteNBO o;
+
+	  o = GNUNET_TIME_absolute_hton (now);
+	  if (sizeof (o) !=
+	      GNUNET_DISK_file_write (fh,
+				      &o,
+				      sizeof (o)))
+	    size = 0;
+	  else
+	    size = sizeof (o);
+	}
+	if (size == sizeof (*map))
+	{
+	  map = GNUNET_DISK_file_map (fh,
+				      &map_handle,
+				      GNUNET_DISK_MAP_TYPE_READWRITE,
+				      sizeof (*map));
+	  if (NULL == map)
+	    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+			_("Failed to map `%s', cannot assure monotonic time!\n"),
+			filename);
+	}
+	else
+	{
+	  GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+		      _("Failed to setup monotonic time file `%s', cannot assure monotonic time!\n"),
+		      filename);
+	}
+      }
+      GNUNET_DISK_file_close (fh);
+      GNUNET_free (filename);
+    }
+  }
+  if (NULL != map)
+  {
+    struct GNUNET_TIME_AbsoluteNBO mt;
+
+#if __STDC_NO_ATOMICS__
+#if __GNUC__
+    mt.abs_value_us__ = __sync_fetch_and_or (map, 0);
+#else
+    mt.abs_value_us__ = *map; /* godspeed, pray this is atomic */
+#endif
+#else
+    mt.abs_value_us__ = atomic_load (map);
+#endif
+    last_time = GNUNET_TIME_absolute_max (GNUNET_TIME_absolute_ntoh (mt),
+					  last_time);
+  }
+  if (now.abs_value_us <= last_time.abs_value_us)
+    now.abs_value_us = last_time.abs_value_us+1;
+  last_time = now;
+  if (NULL != map)
+  {
+    uint64_t val = GNUNET_TIME_absolute_hton (now).abs_value_us__;
+#if __STDC_NO_ATOMICS__
+#if __GNUC__
+    (void) __sync_lock_test_and_set (map, val);
+#else
+    *map = val;  /* godspeed, pray this is atomic */
+#endif
+#else
+    atomic_store (map,
+		  val);
+#endif
+  }
+  return now;
+}
+
+
+/**
+ * Destructor
+ */
+void __attribute__ ((destructor))
+GNUNET_util_time_fini ()
+{
+  (void) GNUNET_TIME_absolute_get_monotonic (NULL);
+}
 
 /* end of time.c */
